# frozen_string_literal: true

module SpiritHub
  class TagsetsController < ApplicationController
    include AddAction

    def logs
      skip_authorization
      skip_policy_scope

      @tagset = Tagset.find(params[:id])
      @logs = @tagset.logs

      respond_to(&:html)
    end

    def doc
      skip_authorization
      skip_policy_scope

      @tagset = Tagset.find(params[:id])
      @logs = @tagset.logs
      @doc = @tagset.docs.find_by(id: params[:doc_id])

      respond_to(&:html)
    end

    def add_doc_form
      skip_authorization
      skip_policy_scope

      @tagsets = Tagset.where(type: 'directory').order_by(updated_at: :desc).limit(5)

      render turbo_stream: [
        turbo_stream.update(:modal, partial: 'spirit_hub/tagsets/add_doc_form')
      ]
    end

    def add_doc
      skip_authorization
      skip_policy_scope

      model = Tagset.create_tagset_doc_from_file(params[:files], @user)
      model.add_tagset_doc_to_object(params[:tagset_id], @user)

      flash.now[:notice] = 'Documents ajouté'
      update_flash_message

      turbo_action_for_add_doc(model)
      add_action(model, params)

      render turbo_stream: @turbo_actions
    end

    def share_form
      skip_authorization
      skip_policy_scope

      model = Tagset.find(params[:id])

      render turbo_stream: [generate_share_form(model)]
    end

    def share
      skip_authorization
      skip_policy_scope

      model = Tagset.find(params[:id])
      model.share_to_users(**share_params)

      @turbo_actions << turbo_stream.replace("informed_badges_#{model.id}", partial: 'spirit_hub/common/informed_badges', locals: { model: })

      flash.now[:notice] = 'Document partagé'
      update_flash_message

      update_turbo_action_current_state(model)

      render turbo_stream: @turbo_actions
    end

    def classify
      skip_authorization
      skip_policy_scope

      ids = params[:ids].split(',')

      models = Tagset.where(:id.in => ids).to_a

      models.each do |model|
        model.add_tagset_doc_to_object(params[:tagset_id], @user)
      end

      if route_controller_from_referer == 'spirit_hub/directories'
        @turbo_actions = [turbo_stream.reload]
      else
        flash.now[:notice] = 'Documents classés'
        update_flash_message
        classify_turbo_actions(models)
        add_action(models, params)
      end

      render turbo_stream: @turbo_actions
    end

    def search_directories
      skip_authorization
      skip_policy_scope

      @tagsets = Tagset.where(type: 'directory', :operation_id.ne => nil, title: /#{Regexp.escape(params[:query]).accent_to_regexp}/i).order_by(updated_at: :desc).limit(5)

      respond_to do |format|
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.replace(:directory_selector, partial: 'spirit_hub/tagsets/directory_selector', locals: { tagsets: @tagsets })
          ]
        end
      end
    end

    def add_favorite
      skip_authorization
      skip_policy_scope

      @model = Tagset.find(params[:id])

      if Favorite.create(user: @user, tagset: @model)
        set_user_favorites
        flash.now[:notice] = 'Ajouté aux favoris'

        render turbo_stream: [
          turbo_stream.replace("add_remove_favorites_#{@model.id}", partial: 'spirit_hub/common/add_remove_favorites', locals: { model: @model }),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      else
        flash.now[:alert] = 'Erreur lors de l\'ajout au favoris'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def remove_favorite
      skip_authorization
      skip_policy_scope

      @model = Tagset.find(params[:id])

      if Favorite.where(user: @user, tagset: @model).destroy_all
        set_user_favorites
        flash.now[:notice] = 'Supprimé des favoris'

        render turbo_stream: [
          turbo_stream.replace("add_remove_favorites_#{@model.id}", partial: 'spirit_hub/common/add_remove_favorites', locals: { model: @model }),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      else
        flash.now[:alert] = 'Erreur lors de l\'ajout au favoris'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def delete_item
      skip_authorization
      skip_policy_scope

      @model = Tagset.find(params[:id])

      success_message = @model.doc? ? 'Document supprimé' : 'Dossier supprimé'

      @model.current_user_id = @user.id

      if @model.destroy
        flash.now[:notice] = success_message

        render turbo_stream: [
          turbo_stream.remove("tagset_#{params[:id]}"),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
          turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -1 })
        ]
      else
        flash.now[:alert] = 'Erreur lors de la suppresion'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def bulk_delete
      skip_authorization
      skip_policy_scope

      ids = params[:ids].split(',')

      Tagset.where(:id.in => ids).each do |tagset|
        tagset.current_user_id = @user.id
        tagset.destroy
      end

      flash.now[:notice] = 'Documents supprimés'

      render turbo_stream: [
        *ids.map { |id| turbo_stream.remove("tagset_#{id}") },
        turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
        turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -ids.length })
      ]
    end

    def form
      skip_authorization
      skip_policy_scope

      models = Tagset.where(:id.in => params[:ids].split(',')).to_a

      generate_action_form(params[:state_key].to_sym, models)

      render turbo_stream: @turbo_actions
    end

    private

    def share_params
      params.to_unsafe_h.symbolize_keys
            .slice(:to, :cc, :bcc, :subject, :message)
            .merge(current_user: @user)
            .transform_values!(&:presence).compact
    end

    def classify_turbo_actions(models)
      case route_controller_from_referer
      when 'spirit_hub/to_classify'
        models.each do |model|
          @turbo_actions << turbo_stream.remove("tagset_#{model.id}")
        end
        @turbo_actions << turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -models.length })
      else
        models.each do |model|
          update_turbo_action_current_state(model)
        end
      end
    end

    def turbo_action_for_add_doc(model)
      case route_controller_from_referer
      when 'spirit_hub/favorites'
        @counts = @user.dashboard_counts
        @turbo_actions << turbo_stream.replace('favorites-counts', partial: 'spirit_hub/favorites/counts', formats: [:html])
      when 'spirit_hub/todos'
        handle_todos_add_doc(model)
      when 'spirit_hub/to_classify'
        handle_to_classify_add_doc(model)
      else
        []
      end
    end

    def handle_todos_add_doc(model)
      return [] if model.reload.parent_tagset_ids.blank?

      # TODO : Refactor this
      referer_controller = SpiritHub::TodosController.new
      referer_controller.instance_variable_set(:@user, @user)
      referer_controller.set_columns
      @user_column_names = referer_controller.instance_variable_get(:@user_column_names)
      @user_favorites = []

      partial = referer_controller.set_view_type == 'table' ? 'spirit_hub/todos/row' : 'spirit_hub/todos/card'
      @turbo_actions << turbo_stream.prepend('todos-table-body', partial:, formats: [:html], locals: { model: })
    end

    def handle_to_classify_add_doc(model)
      return [] unless model.reload.parent_tagset_ids.blank?

      # TODO : Refactor this
      referer_controller = SpiritHub::TodosController.new
      referer_controller.instance_variable_set(:@user, @user)
      referer_controller.set_columns
      @user_column_names = referer_controller.instance_variable_get(:@user_column_names)
      @user_favorites = []

      partial = referer_controller.set_view_type == 'table' ? 'spirit_hub/to_classify/row' : 'spirit_hub/to_classify/card'
      @turbo_actions << turbo_stream.prepend('to-classify-table-body', partial:, formats: [:html], locals: { model: })
    end
  end
end
