# frozen_string_literal: true

require 'rails_helper'

RSpec.describe SpiritHub::TagsetShare, type: :service do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  subject { described_class.call(**params) }

  let(:params) do
    {
      tagset:,
      to: [user1.id.to_s],
      cc: [user2.id.to_s],
      bcc: [user3.id.to_s],
      subject: message_subject,
      message:,
      current_user: user4
    }
  end

  let(:tagset) { create(:tagset_doc, :with_file) }

  let(:user1) { create(:user) }
  let(:user2) { create(:user) }
  let(:user3) { create(:user) }
  let(:user4) { create(:user) }

  let(:message_subject) { Faker::Lorem.sentence }
  let(:message) { Faker::Lorem.sentence }

  let(:share_email) { double(deliver: nil) }

  before do
    allow(SpiritHub::TagsetsMailer).to receive(:share).and_return(share_email)
  end

  it 'shares the doc to the user' do
    # creates an Upload record
    expect { subject }.to change(Upload, :count).by(1)

    # updates the informed fields accordingly
    expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id])
    expect(tagset.informed_main).to match_array([user1.id])
    expect(tagset.informed_cc).to match_array([user2.id])
    expect(tagset.informed_bcc).to match_array([user3.id])

    expect(tagset.current_state).to eq('Partagé')
    expect(tagset.state.key).to eq('share')

    # sends the share email
    expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
      upload: instance_of(Upload),
      subject: message_subject,
      message:,
      to: [user1.email],
      cc: [user2.email],
      bcc: [user3.email]
    )
    expect(share_email).to have_received(:deliver)

    # logs the action
    expect(Log.count).to eq(1)
  end

  context 'when passing a group as one of the main recipients' do
    before { params[:to] += [group_id_prefixed] }

    let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
    let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

    let(:user5) { create(:user) }
    let(:user6) { create(:user) }

    it 'shares the doc with the users within that group' do
      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
      expect(tagset.informed_main).to match_array([user1.id, user5.id, user6.id])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          to: [user1.email, user5.email, user6.email]
        )
      )
    end
  end

  context 'when passing a group as one of the CC recipients' do
    before { params[:cc] += [group_id_prefixed] }

    let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
    let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

    let(:user5) { create(:user) }
    let(:user6) { create(:user) }

    it 'shares the doc with the users within that group' do
      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
      expect(tagset.informed_cc).to match_array([user2.id, user5.id, user6.id])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          cc: [user2.email, user5.email, user6.email]
        )
      )
    end
  end

  context 'when passing a group as one of the BCC recipients' do
    before { params[:bcc] += [group_id_prefixed] }

    let(:group_id_prefixed) { [described_class::GROUP_PREFIX, group.id].join }
    let(:group) { create(:group, users_ids: [user5.id, user6.id]) }

    let(:user5) { create(:user) }
    let(:user6) { create(:user) }

    it 'shares the doc with the users within that group' do
      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, user5.id, user6.id])
      expect(tagset.informed_bcc).to match_array([user3.id, user5.id, user6.id])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          bcc: [user3.email, user5.email, user6.email]
        )
      )
    end
  end

  context 'when passing a contact as one of the main recipients' do
    before { params[:to] += [contact_id_prefixed] }

    let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
    let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
    let(:contact) { create(:contact) }

    it 'shares the doc with that contact' do
      expect(contact.reload.email).to be_present

      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
      expect(tagset.informed_main).to match_array([user1.id, contact_id_tagged])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          to: [user1.email, contact.email]
        )
      )
    end
  end

  context 'when passing a contact as one of the CC recipients' do
    before { params[:cc] += [contact_id_prefixed] }

    let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
    let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
    let(:contact) { create(:contact) }

    it 'shares the doc with that contact' do
      expect(contact.reload.email).to be_present

      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
      expect(tagset.informed_cc).to match_array([user2.id, contact_id_tagged])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          cc: [user2.email, contact.email]
        )
      )
    end
  end

  context 'when passing a contact as one of the BCC recipients' do
    before { params[:bcc] += [contact_id_prefixed] }

    let(:contact_id_prefixed) { [described_class::CONTACT_PREFIX, contact.id].join }
    let(:contact_id_tagged) { "#{contact.class}##{contact.id}" }
    let(:contact) { create(:contact) }

    it 'shares the doc with that contact' do
      expect(contact.reload.email).to be_present

      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
      expect(tagset.informed_bcc).to match_array([user3.id, contact_id_tagged])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          bcc: [user3.email, contact.email]
        )
      )
    end
  end

  context 'when passing a NEW contact as one of the main recipients' do
    before { params[:to] += ['jack.sparrow@gmail.com'] }

    let(:contact_created) { Doc.where(type: 'contact').last }
    let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

    it 'creates a new Doc(contact) record' do
      expect { subject }.to change { Doc.count }.by(1)

      expect(contact_created).to be_present
      expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.current_user_id).to eq user4.id
    end

    it 'shares the doc with that contact' do
      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
      expect(tagset.informed_main).to match_array([user1.id, contact_id_tagged])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          to: [user1.email, contact_created.email]
        )
      )
    end
  end

  context 'when passing a NEW contact as one of the CC recipients' do
    before { params[:cc] += ['jack.sparrow@gmail.com'] }

    let(:contact_created) { Doc.where(type: 'contact').last }
    let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

    it 'creates a new Doc(contact) record' do
      expect { subject }.to change { Doc.count }.by(1)

      expect(contact_created).to be_present
      expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.current_user_id).to eq user4.id
    end

    it 'shares the doc with that contact' do
      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
      expect(tagset.informed_cc).to match_array([user2.id, contact_id_tagged])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          cc: [user2.email, contact_created.email]
        )
      )
    end
  end

  context 'when passing a NEW contact as one of the BCC recipients' do
    before { params[:bcc] += ['jack.sparrow@gmail.com'] }

    let(:contact_created) { Doc.where(type: 'contact').last }
    let(:contact_id_tagged) { "#{contact_created.class}##{contact_created.id}" }

    it 'creates a new Doc(contact) record' do
      expect { subject }.to change { Doc.count }.by(1)

      expect(contact_created).to be_present
      expect(contact_created.email).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.title).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.superfields['email_address_1']).to eq 'jack.sparrow@gmail.com'
      expect(contact_created.current_user_id).to eq user4.id
    end

    it 'shares the doc with that contact' do
      subject

      # updates the informed fields accordingly
      expect(tagset.reload.informed).to match_array([user1.id, user2.id, user3.id, contact_id_tagged])
      expect(tagset.informed_bcc).to match_array([user3.id, contact_id_tagged])

      # sends the share email
      expect(SpiritHub::TagsetsMailer).to have_received(:share).with(
        hash_including(
          bcc: [user3.email, contact_created.email]
        )
      )
    end
  end
end
